@@ -35,6 +35,9 @@ pango_attr_type_get_type (void)
         { PANGO_ATTR_ABSOLUTE_SIZE, "PANGO_ATTR_ABSOLUTE_SIZE", "absolute-size" },
         { PANGO_ATTR_GRAVITY, "PANGO_ATTR_GRAVITY", "gravity" },
         { PANGO_ATTR_GRAVITY_HINT, "PANGO_ATTR_GRAVITY_HINT", "gravity-hint" },
+        { PANGO_ATTR_FONT_FEATURES, "PANGO_ATTR_FONT_FEATURES", "font-features" },
+        { PANGO_ATTR_FOREGROUND_ALPHA, "PANGO_ATTR_FOREGROUND_ALPHA", "foreground-alpha" },
+        { PANGO_ATTR_BACKGROUND_ALPHA, "PANGO_ATTR_BACKGROUND_ALPHA", "background-alpha" },
         { 0, NULL, NULL }
       };
       GType g_define_type_id =
@@ -207,6 +210,7 @@ pango_weight_get_type (void)
         { PANGO_WEIGHT_THIN, "PANGO_WEIGHT_THIN", "thin" },
         { PANGO_WEIGHT_ULTRALIGHT, "PANGO_WEIGHT_ULTRALIGHT", "ultralight" },
         { PANGO_WEIGHT_LIGHT, "PANGO_WEIGHT_LIGHT", "light" },
+        { PANGO_WEIGHT_SEMILIGHT, "PANGO_WEIGHT_SEMILIGHT", "semilight" },
         { PANGO_WEIGHT_BOOK, "PANGO_WEIGHT_BOOK", "book" },
         { PANGO_WEIGHT_NORMAL, "PANGO_WEIGHT_NORMAL", "normal" },
         { PANGO_WEIGHT_MEDIUM, "PANGO_WEIGHT_MEDIUM", "medium" },
